# - Find PCRE
# Find the PCRE includes and client library
# This module defines
#  PCRE_INCLUDE_DIRS, where to find mysql.h
#  PCRE_LIBRARIES, the libraries needed to use GeoIP.
#  PCRE_FOUND, If false, do not try to use GeoIP.
#

IF(PCRE_INCLUDE_DIR AND PCRE_LIBRARIES)
	# Already in cache, be silent
	SET(PCRE_FIND_QUIETLY TRUE)
ENDIF(PCRE_INCLUDE_DIR AND PCRE_LIBRARIES)

FIND_PATH(PCRE_INCLUDE_DIR pcre.h
	/usr/include/
	/usr/local/include/
)

FIND_LIBRARY(PCRE_PCRE_LIBRARY NAMES pcre
	PATHS
	/usr/lib
	/usr/local/lib
)

FIND_LIBRARY(PCRE_PCREPOSIX_LIBRARY NAMES pcreposix
	PATHS
	/usr/lib
	/usr/local/lib
)

set(PCRE_LIBRARIES ${PCRE_PCRE_LIBRARY} ${PCRE_PCREPOSIX_LIBRARY} CACHE STRING "The libraries needed to use PCRE")

IF(PCRE_INCLUDE_DIR AND PCRE_LIBRARIES)
	SET(PCRE_FOUND TRUE)
ENDIF(PCRE_INCLUDE_DIR AND PCRE_LIBRARIES)

IF(PCRE_FOUND)
	IF(NOT Pcre_FIND_QUIETLY)
		MESSAGE(STATUS "Found PCRE: ${PCRE_INCLUDE_DIR}, ${PCRE_LIBRARIES}")
	ENDIF(NOT Pcre_FIND_QUIETLY)
ELSE(PCRE_FOUND)
	IF(Pcre_FIND_REQUIRED)
		MESSAGE(FATAL_ERROR "Could NOT find PCRE")
	ENDIF(Pcre_FIND_REQUIRED)
ENDIF(PCRE_FOUND)

MARK_AS_ADVANCED(PCRE_INCLUDE_DIR PCRE_LIBRARIES PCRE_PCREPOSIX_LIBRARY PCRE_PCRE_LIBRARY)
